@@ -29,19 +29,19 @@ import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 
-import javax.servlet.Servlet;
-import javax.servlet.ServletConfig;
-import javax.servlet.ServletContainerInitializer;
-import javax.servlet.ServletContext;
-import javax.servlet.ServletException;
-import javax.servlet.ServletRegistration;
-import javax.servlet.annotation.HttpConstraint;
-import javax.servlet.annotation.HttpMethodConstraint;
-import javax.servlet.annotation.ServletSecurity;
-import javax.servlet.annotation.ServletSecurity.EmptyRoleSemantic;
-import javax.servlet.http.HttpServlet;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpServletResponse;
+import jakarta.servlet.Servlet;
+import jakarta.servlet.ServletConfig;
+import jakarta.servlet.ServletContainerInitializer;
+import jakarta.servlet.ServletContext;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.ServletRegistration;
+import jakarta.servlet.annotation.HttpConstraint;
+import jakarta.servlet.annotation.HttpMethodConstraint;
+import jakarta.servlet.annotation.ServletSecurity;
+import jakarta.servlet.annotation.ServletSecurity.EmptyRoleSemantic;
+import jakarta.servlet.http.HttpServlet;
+import jakarta.servlet.http.HttpServletRequest;
+import jakarta.servlet.http.HttpServletResponse;
 
 import org.junit.Assert;
 import org.junit.Test;
@@ -65,22 +65,26 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
     @Test
     public void testSecurityAnnotationsSubclass1() throws Exception {
-        doTest(SubclassDenyAllServlet.class.getName(), false, false, false, false);
+        doTest(SubclassDenyAllServlet.class.getName(),
+                false, false, false,false);
     }
 
     @Test
     public void testSecurityAnnotationsSubclass2() throws Exception {
-        doTest(SubclassAllowAllServlet.class.getName(), false, false, true, false);
+        doTest(SubclassAllowAllServlet.class.getName(),
+                false, false, true, false);
     }
 
     @Test
     public void testSecurityAnnotationsMethods1() throws Exception {
-        doTest(MethodConstraintServlet.class.getName(), false, false, false, false);
+        doTest(MethodConstraintServlet.class.getName(),
+                false, false, false, false);
     }
 
     @Test
     public void testSecurityAnnotationsMethods2() throws Exception {
-        doTest(MethodConstraintServlet.class.getName(), true, false, true, false);
+        doTest(MethodConstraintServlet.class.getName(),
+                true, false, true, false);
     }
 
     @Test
@@ -155,8 +159,9 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         ByteChunk bc = new ByteChunk();
         int rc;
-        rc = getUrl("http://localhost:" + getPort() + "/testStandardWrapper/securityAnnotationsWebXmlPriority", bc,
-                null, null);
+        rc = getUrl("http://localhost:" + getPort() +
+                "/testStandardWrapper/securityAnnotationsWebXmlPriority",
+                bc, null, null);
 
         Assert.assertTrue(bc.getLength() > 0);
         Assert.assertEquals(403, rc);
@@ -168,7 +173,8 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         ByteChunk bc = new ByteChunk();
         int rc;
-        rc = getUrl("http://localhost:" + getPort() + "/test/testStandardWrapper/securityAnnotationsMetaDataPriority",
+        rc = getUrl("http://localhost:" + getPort() +
+                "/test/testStandardWrapper/securityAnnotationsMetaDataPriority",
                 bc, null, null);
 
         Assert.assertEquals("OK", bc.toString());
@@ -197,7 +203,8 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         ByteChunk bc = new ByteChunk();
         int rc;
-        rc = getUrl("http://localhost:" + getPort() + "/", bc, null, null);
+        rc = getUrl("http://localhost:" + getPort() + "/",
+                bc, null, null);
 
         Assert.assertTrue(bc.getLength() > 0);
         Assert.assertEquals(403, rc);
@@ -215,14 +222,16 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         ByteChunk bc = new ByteChunk();
         int rc;
-        rc = getUrl("http://localhost:" + getPort() + "/protected.jsp", bc, null, null);
+        rc = getUrl("http://localhost:" + getPort() + "/protected.jsp",
+                bc, null, null);
 
         Assert.assertTrue(bc.getLength() > 0);
         Assert.assertEquals(403, rc);
 
         bc.recycle();
 
-        rc = getUrl("http://localhost:" + getPort() + "/unprotected.jsp", bc, null, null);
+        rc = getUrl("http://localhost:" + getPort() + "/unprotected.jsp",
+                bc, null, null);
 
         Assert.assertEquals(200, rc);
         Assert.assertTrue(bc.toString().contains("00-OK"));
@@ -243,12 +252,13 @@ public class TestStandardWrapper extends TomcatBaseTest {
         doTestRoleMapping("context");
     }
 
-    private void doTestRoleMapping(String realmContainer) throws Exception {
+    private void doTestRoleMapping(String realmContainer)
+            throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = getProgrammaticRootContext();
+        Context ctx = tomcat.addContext("", null);
         ctx.addRoleMapping("testRole", "very-complex-role-name");
 
         Wrapper wrapper = Tomcat.addServlet(ctx, "servlet", RoleAllowServlet.class.getName());
@@ -262,9 +272,8 @@ public class TestStandardWrapper extends TomcatBaseTest {
         ch.setAlgorithm("SHA");
         realm.setCredentialHandler(ch);
 
-        /*
-         * Attach the realm to the appropriate container, but role mapping must always succeed because it is evaluated
-         * at context level.
+        /* Attach the realm to the appropriate container, but role mapping must
+         * always succeed because it is evaluated at context level.
          */
         if (realmContainer.equals("engine")) {
             tomcat.getEngine().setRealm(realm);
@@ -297,25 +306,27 @@ public class TestStandardWrapper extends TomcatBaseTest {
         // This now tests RealmBase#hasResourcePermission() because we need a wrapper
         // to be passed from an authenticator
         ByteChunk bc = new ByteChunk();
-        Map<String,List<String>> reqHeaders = new HashMap<>();
+        Map<String, List<String>> reqHeaders = new HashMap<>();
         List<String> authHeaders = new ArrayList<>();
         // testUser, testPwd
         authHeaders.add("Basic dGVzdFVzZXI6dGVzdFB3ZA==");
         reqHeaders.put("Authorization", authHeaders);
 
-        int rc = getUrl("http://localhost:" + getPort() + "/", bc, reqHeaders, null);
+        int rc = getUrl("http://localhost:" + getPort() + "/", bc, reqHeaders,
+                null);
 
         Assert.assertEquals("OK", bc.toString());
         Assert.assertEquals(200, rc);
     }
 
-    private void doTestSecurityAnnotationsAddServlet(boolean useCreateServlet) throws Exception {
+    private void doTestSecurityAnnotationsAddServlet(boolean useCreateServlet)
+            throws Exception {
 
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = getProgrammaticRootContext();
+        Context ctx = tomcat.addContext("", null);
 
         Servlet s = new DenyAllServlet();
         ServletContainerInitializer sci = new SCI(s, useCreateServlet);
@@ -336,14 +347,15 @@ public class TestStandardWrapper extends TomcatBaseTest {
         }
     }
 
-    private void doTest(String servletClassName, boolean usePost, boolean useRole, boolean expect200,
-            boolean denyUncovered) throws Exception {
+    private void doTest(String servletClassName, boolean usePost,
+            boolean useRole, boolean expect200, boolean denyUncovered)
+            throws Exception {
 
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = getProgrammaticRootContext();
+        Context ctx = tomcat.addContext("", null);
 
         ctx.setDenyUncoveredHttpMethods(denyUncovered);
 
@@ -375,9 +387,11 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         int rc;
         if (usePost) {
-            rc = postUrl(null, "http://localhost:" + getPort() + "/", bc, reqHeaders, null);
+            rc = postUrl(null, "http://localhost:" + getPort() + "/", bc,
+                    reqHeaders, null);
         } else {
-            rc = getUrl("http://localhost:" + getPort() + "/", bc, reqHeaders, null);
+            rc = getUrl("http://localhost:" + getPort() + "/", bc, reqHeaders,
+                    null);
         }
 
         if (expect200) {
@@ -393,14 +407,16 @@ public class TestStandardWrapper extends TomcatBaseTest {
         private static final long serialVersionUID = 1L;
 
         @Override
-        protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
+        protected void doGet(HttpServletRequest req, HttpServletResponse resp)
+                throws ServletException, IOException {
 
             resp.setContentType("text/plain");
             resp.getWriter().print("OK");
         }
 
         @Override
-        protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
+        protected void doPost(HttpServletRequest req, HttpServletResponse resp)
+                throws ServletException, IOException {
             doGet(req, resp);
         }
     }
@@ -419,13 +435,20 @@ public class TestStandardWrapper extends TomcatBaseTest {
         private static final long serialVersionUID = 1L;
     }
 
-    @ServletSecurity(value = @HttpConstraint(EmptyRoleSemantic.PERMIT), httpMethodConstraints = {
-            @HttpMethodConstraint(value = "GET", emptyRoleSemantic = EmptyRoleSemantic.DENY) })
+    @ServletSecurity(value= @HttpConstraint(EmptyRoleSemantic.PERMIT),
+        httpMethodConstraints = {
+            @HttpMethodConstraint(value="GET",
+                    emptyRoleSemantic = EmptyRoleSemantic.DENY)
+        }
+    )
     public static class MethodConstraintServlet extends TestServlet {
         private static final long serialVersionUID = 1L;
     }
 
-    @ServletSecurity(httpMethodConstraints = { @HttpMethodConstraint(value = "POST", rolesAllowed = "testRole") })
+    @ServletSecurity(httpMethodConstraints = {
+            @HttpMethodConstraint(value="POST",rolesAllowed = "testRole")
+        }
+    )
     public static class UncoveredGetServlet extends TestServlet {
         private static final long serialVersionUID = 1L;
     }
@@ -451,7 +474,8 @@ public class TestStandardWrapper extends TomcatBaseTest {
         }
 
         @Override
-        public void onStartup(Set<Class<?>> c, ServletContext ctx) throws ServletException {
+        public void onStartup(Set<Class<?>> c, ServletContext ctx)
+                throws ServletException {
             Servlet s;
 
             if (createServlet) {
@@ -491,24 +515,24 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         // Start the threads
         Bug51445Thread[] threads = new Bug51445Thread[5];
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i++) {
+        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
             threads[i] = new Bug51445Thread(getPort());
             threads[i].start();
         }
 
         // Wait for threads to finish
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i++) {
+        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
             threads[i].join();
         }
 
         Set<String> servlets = new HashSet<>();
         // Output the result
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i++) {
+        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
             System.out.println(threads[i].getResult());
         }
 
         // Check the result
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i++) {
+        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
             String[] results = threads[i].getResult().split(",");
             Assert.assertEquals(2, results.length);
             Assert.assertEquals("10", results[0]);
@@ -537,23 +561,23 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         // Start the threads
         Bug51445Thread[] threads = new Bug51445Thread[5];
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i++) {
+        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
             threads[i] = new Bug51445Thread(getPort());
             threads[i].start();
         }
 
         // Wait for threads to finish
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i++) {
+        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
             threads[i].join();
         }
 
         Set<String> servlets = new HashSet<>();
         // Output the result
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i++) {
+        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
             System.out.println(threads[i].getResult());
         }
         // Check the result
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i++) {
+        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
             String[] results = threads[i].getResult().split(",");
             Assert.assertEquals(2, results.length);
             Assert.assertEquals("10", results[0]);
@@ -567,7 +591,7 @@ public class TestStandardWrapper extends TomcatBaseTest {
         private int port;
         private String result;
 
-        Bug51445Thread(int port) {
+        public Bug51445Thread(int port) {
             this.port = port;
         }
 
@@ -590,7 +614,8 @@ public class TestStandardWrapper extends TomcatBaseTest {
      * SingleThreadModel servlet that sets a value in the init() method.
      */
     @SuppressWarnings("deprecation")
-    public static class Bug51445Servlet extends HttpServlet implements javax.servlet.SingleThreadModel {
+    public static class Bug51445Servlet extends HttpServlet
+            implements jakarta.servlet.SingleThreadModel {
 
         private static final long serialVersionUID = 1L;
         private static final long LATCH_TIMEOUT = 60;
@@ -606,7 +631,8 @@ public class TestStandardWrapper extends TomcatBaseTest {
         }
 
         @Override
-        protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
+        protected void doGet(HttpServletRequest req, HttpServletResponse resp)
+                throws ServletException, IOException {
 
             boolean latchAwaitResult = false;
 
